var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __commonJS = (cb, mod) => function __require() {
  return mod || (0, cb[__getOwnPropNames(cb)[0]])((mod = { exports: {} }).exports, mod), mod.exports;
};
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// rust-module.js
var require_rust_module = __commonJS({
  "rust-module.js"(exports, module2) {
    var { existsSync, readFileSync } = require("fs");
    var { join } = require("path");
    var { platform, arch } = process;
    var nativeBinding = null;
    var localFileExisted = false;
    var loadError = null;
    function isMusl() {
      if (!process.report || typeof process.report.getReport !== "function") {
        try {
          const lddPath = require("child_process").execSync("which ldd").toString().trim();
          return readFileSync(lddPath, "utf8").includes("musl");
        } catch (e) {
          return true;
        }
      } else {
        const { glibcVersionRuntime } = process.report.getReport().header;
        return !glibcVersionRuntime;
      }
    }
    switch (platform) {
      case "android":
        switch (arch) {
          case "arm64":
            localFileExisted = existsSync(join(__dirname, "index.android-arm64.node"));
            try {
              if (localFileExisted) {
                nativeBinding = require("./index.android-arm64.node");
              } else {
                nativeBinding = require("slint-ui-android-arm64");
              }
            } catch (e) {
              loadError = e;
            }
            break;
          case "arm":
            localFileExisted = existsSync(join(__dirname, "index.android-arm-eabi.node"));
            try {
              if (localFileExisted) {
                nativeBinding = require("./index.android-arm-eabi.node");
              } else {
                nativeBinding = require("slint-ui-android-arm-eabi");
              }
            } catch (e) {
              loadError = e;
            }
            break;
          default:
            throw new Error(`Unsupported architecture on Android ${arch}`);
        }
        break;
      case "win32":
        switch (arch) {
          case "x64":
            localFileExisted = existsSync(
              join(__dirname, "index.win32-x64-msvc.node")
            );
            try {
              if (localFileExisted) {
                nativeBinding = require("./index.win32-x64-msvc.node");
              } else {
                nativeBinding = require("slint-ui-win32-x64-msvc");
              }
            } catch (e) {
              loadError = e;
            }
            break;
          case "ia32":
            localFileExisted = existsSync(
              join(__dirname, "index.win32-ia32-msvc.node")
            );
            try {
              if (localFileExisted) {
                nativeBinding = require("./index.win32-ia32-msvc.node");
              } else {
                nativeBinding = require("slint-ui-win32-ia32-msvc");
              }
            } catch (e) {
              loadError = e;
            }
            break;
          case "arm64":
            localFileExisted = existsSync(
              join(__dirname, "index.win32-arm64-msvc.node")
            );
            try {
              if (localFileExisted) {
                nativeBinding = require("./index.win32-arm64-msvc.node");
              } else {
                nativeBinding = require("slint-ui-win32-arm64-msvc");
              }
            } catch (e) {
              loadError = e;
            }
            break;
          default:
            throw new Error(`Unsupported architecture on Windows: ${arch}`);
        }
        break;
      case "darwin":
        localFileExisted = existsSync(join(__dirname, "index.darwin-universal.node"));
        try {
          if (localFileExisted) {
            nativeBinding = require("./index.darwin-universal.node");
          } else {
            nativeBinding = require("slint-ui-darwin-universal");
          }
          break;
        } catch {
        }
        switch (arch) {
          case "x64":
            localFileExisted = existsSync(join(__dirname, "index.darwin-x64.node"));
            try {
              if (localFileExisted) {
                nativeBinding = require("./index.darwin-x64.node");
              } else {
                nativeBinding = require("slint-ui-darwin-x64");
              }
            } catch (e) {
              loadError = e;
            }
            break;
          case "arm64":
            localFileExisted = existsSync(
              join(__dirname, "index.darwin-arm64.node")
            );
            try {
              if (localFileExisted) {
                nativeBinding = require("./index.darwin-arm64.node");
              } else {
                nativeBinding = require("slint-ui-darwin-arm64");
              }
            } catch (e) {
              loadError = e;
            }
            break;
          default:
            throw new Error(`Unsupported architecture on macOS: ${arch}`);
        }
        break;
      case "freebsd":
        if (arch !== "x64") {
          throw new Error(`Unsupported architecture on FreeBSD: ${arch}`);
        }
        localFileExisted = existsSync(join(__dirname, "index.freebsd-x64.node"));
        try {
          if (localFileExisted) {
            nativeBinding = require("./index.freebsd-x64.node");
          } else {
            nativeBinding = require("slint-ui-freebsd-x64");
          }
        } catch (e) {
          loadError = e;
        }
        break;
      case "linux":
        switch (arch) {
          case "x64":
            if (isMusl()) {
              localFileExisted = existsSync(
                join(__dirname, "index.linux-x64-musl.node")
              );
              try {
                if (localFileExisted) {
                  nativeBinding = require("./index.linux-x64-musl.node");
                } else {
                  nativeBinding = require("slint-ui-linux-x64-musl");
                }
              } catch (e) {
                loadError = e;
              }
            } else {
              localFileExisted = existsSync(
                join(__dirname, "index.linux-x64-gnu.node")
              );
              try {
                if (localFileExisted) {
                  nativeBinding = require("./index.linux-x64-gnu.node");
                } else {
                  nativeBinding = require("slint-ui-linux-x64-gnu");
                }
              } catch (e) {
                loadError = e;
              }
            }
            break;
          case "arm64":
            if (isMusl()) {
              localFileExisted = existsSync(
                join(__dirname, "index.linux-arm64-musl.node")
              );
              try {
                if (localFileExisted) {
                  nativeBinding = require("./index.linux-arm64-musl.node");
                } else {
                  nativeBinding = require("slint-ui-linux-arm64-musl");
                }
              } catch (e) {
                loadError = e;
              }
            } else {
              localFileExisted = existsSync(
                join(__dirname, "index.linux-arm64-gnu.node")
              );
              try {
                if (localFileExisted) {
                  nativeBinding = require("./index.linux-arm64-gnu.node");
                } else {
                  nativeBinding = require("slint-ui-linux-arm64-gnu");
                }
              } catch (e) {
                loadError = e;
              }
            }
            break;
          case "arm":
            localFileExisted = existsSync(
              join(__dirname, "index.linux-arm-gnueabihf.node")
            );
            try {
              if (localFileExisted) {
                nativeBinding = require("./index.linux-arm-gnueabihf.node");
              } else {
                nativeBinding = require("slint-ui-linux-arm-gnueabihf");
              }
            } catch (e) {
              loadError = e;
            }
            break;
          default:
            throw new Error(`Unsupported architecture on Linux: ${arch}`);
        }
        break;
      default:
        throw new Error(`Unsupported OS: ${platform}, architecture: ${arch}`);
    }
    if (!nativeBinding) {
      if (loadError) {
        throw loadError;
      }
      throw new Error(`Failed to load native binding`);
    }
    var { DiagnosticLevel: DiagnosticLevel3, ComponentCompiler: ComponentCompiler2, ComponentDefinition: ComponentDefinition2, ComponentInstance: ComponentInstance2, ValueType: ValueType2, Property, Window: Window2, SlintRgbaColor: SlintRgbaColor2, SlintBrush: SlintBrush2, SlintImageData: SlintImageData2, SlintModelNotify, SlintPoint: SlintPoint2, SlintSize: SlintSize2, mockElapsedTime: mockElapsedTime2, ProcessEventsResult: ProcessEventsResult2, processEvents: processEvents2, invokeFromEventLoop: invokeFromEventLoop2 } = nativeBinding;
    module2.exports.DiagnosticLevel = DiagnosticLevel3;
    module2.exports.ComponentCompiler = ComponentCompiler2;
    module2.exports.ComponentDefinition = ComponentDefinition2;
    module2.exports.ComponentInstance = ComponentInstance2;
    module2.exports.ValueType = ValueType2;
    module2.exports.Property = Property;
    module2.exports.Window = Window2;
    module2.exports.SlintRgbaColor = SlintRgbaColor2;
    module2.exports.SlintBrush = SlintBrush2;
    module2.exports.SlintImageData = SlintImageData2;
    module2.exports.SlintModelNotify = SlintModelNotify;
    module2.exports.SlintPoint = SlintPoint2;
    module2.exports.SlintSize = SlintSize2;
    module2.exports.mockElapsedTime = mockElapsedTime2;
    module2.exports.ProcessEventsResult = ProcessEventsResult2;
    module2.exports.processEvents = processEvents2;
    module2.exports.invokeFromEventLoop = invokeFromEventLoop2;
  }
});

// index.ts
var node_exports = {};
__export(node_exports, {
  ArrayModel: () => ArrayModel,
  Brush: () => import_rust_module.Brush,
  CompileError: () => CompileError,
  Diagnostic: () => import_rust_module.Diagnostic,
  DiagnosticLevel: () => import_rust_module.DiagnosticLevel,
  Model: () => Model,
  RgbaColor: () => import_rust_module.RgbaColor,
  loadFile: () => loadFile,
  private_api: () => private_api,
  quitEventLoop: () => quitEventLoop,
  runEventLoop: () => runEventLoop
});
module.exports = __toCommonJS(node_exports);
var napi = __toESM(require_rust_module());
var import_rust_module = __toESM(require_rust_module());
var Model = class {
  /**
   * @hidden
   */
  notify;
  constructor() {
    this.notify = new NullPeer();
  }
  /**
   * Notifies the view that the data of the current row is changed.
   * @param row index of the changed row.
   */
  notifyRowDataChanged(row) {
    this.notify.rowDataChanged(row);
  }
  /**
   * Notifies the view that multiple rows are added to the model.
   * @param row index of the first added row.
   * @param count the number of added items.
  */
  notifyRowAdded(row, count) {
    this.notify.rowAdded(row, count);
  }
  /**
   * Notifies the view that multiple rows are removed to the model.
   * @param row index of the first removed row.
   * @param count the number of removed items.
  */
  notifyRowRemoved(row, count) {
    this.notify.rowRemoved(row, count);
  }
  /**
   * Notifies the view that the complete data must be reload.
   */
  notifyReset() {
    this.notify.reset();
  }
};
var NullPeer = class {
  rowDataChanged(row) {
  }
  rowAdded(row, count) {
  }
  rowRemoved(row, count) {
  }
  reset() {
  }
};
var ArrayModel = class extends Model {
  /**
   * @hidden
   */
  #array;
  /**
   * Creates a new ArrayModel.
   *
   * @param arr
   */
  constructor(arr) {
    super();
    this.#array = arr;
  }
  /**
   * Returns the number of entries in the array model.
   */
  get length() {
    return this.#array.length;
  }
  /**
   * Returns the number of entries in the array model.
   */
  rowCount() {
    return this.#array.length;
  }
  /**
   * Returns the data at the specified row.
   * @param row index in range 0..(rowCount() - 1).
   * @returns undefined if row is out of range otherwise the data.
   */
  rowData(row) {
    return this.#array[row];
  }
  /**
   * Stores the given data on the given row index and notifies run-time about the changed row.
   * @param row index in range 0..(rowCount() - 1).
   * @param data new data item to store on the given row index
   */
  setRowData(row, data) {
    this.#array[row] = data;
    this.notifyRowDataChanged(row);
  }
  /**
   * Pushes new values to the array that's backing the model and notifies
   * the run-time about the added rows.
   * @param values list of values that will be pushed to the array.
   */
  push(...values) {
    let size = this.#array.length;
    Array.prototype.push.apply(this.#array, values);
    this.notifyRowAdded(size, arguments.length);
  }
  // FIXME: should this be named splice and have the splice api?
  /**
   * Removes the specified number of element from the array that's backing
   * the model, starting at the specified index.
   * @param index index of first row to remove.
   * @param size number of rows to remove.
   */
  remove(index, size) {
    let r = this.#array.splice(index, size);
    this.notifyRowRemoved(index, size);
  }
  /**
   * Returns an iterable of values in the array.
   */
  values() {
    return this.#array.values();
  }
  /**
   * Returns an iterable of key, value pairs for every entry in the array.
   */
  entries() {
    return this.#array.entries();
  }
};
var Component = class {
  #instance;
  /**
   * @hidden
   */
  constructor(instance) {
    this.#instance = instance;
  }
  get window() {
    return this.#instance.window();
  }
  /**
  * @hidden
  */
  get component_instance() {
    return this.#instance;
  }
  async run() {
    this.show();
    await runEventLoop();
    this.hide();
  }
  show() {
    this.#instance.window().show();
  }
  hide() {
    this.#instance.window().hide();
  }
};
var CompileError = class extends Error {
  /**
   * List of {@link Diagnostic} items emitted while compiling .slint code.
   */
  diagnostics;
  /**
   * Creates a new CompileError.
   *
   * @param message human-readable description of the error.
   * @param diagnostics represent a list of diagnostic items emitted while compiling .slint code.
   */
  constructor(message, diagnostics) {
    super(message);
    this.diagnostics = diagnostics;
  }
};
function loadFile(filePath, options) {
  let compiler = new napi.ComponentCompiler();
  if (typeof options !== "undefined") {
    if (typeof options.style !== "undefined") {
      compiler.style = options.style;
    }
    if (typeof options.includePaths !== "undefined") {
      compiler.includePaths = options.includePaths;
    }
    if (typeof options.libraryPaths !== "undefined") {
      compiler.libraryPaths = options.libraryPaths;
    }
  }
  let definition = compiler.buildFromPath(filePath);
  let diagnostics = compiler.diagnostics;
  if (diagnostics.length > 0) {
    let warnings = diagnostics.filter(
      (d) => d.level == napi.DiagnosticLevel.Warning
    );
    if (typeof options !== "undefined" && options.quiet !== true) {
      warnings.forEach((w) => console.warn("Warning: " + w));
    }
    let errors = diagnostics.filter(
      (d) => d.level == napi.DiagnosticLevel.Error
    );
    if (errors.length > 0) {
      throw new CompileError("Could not compile " + filePath, errors);
    }
  }
  let slint_module = /* @__PURE__ */ Object.create({});
  Object.defineProperty(slint_module, definition.name.replace(/-/g, "_"), {
    value: function(properties) {
      let instance = definition.create();
      if (instance == null) {
        throw Error(
          "Could not create a component handle for" + filePath
        );
      }
      for (var key in properties) {
        let value = properties[key];
        if (value instanceof Function) {
          instance.setCallback(key, value);
        } else {
          instance.setProperty(key, properties[key]);
        }
      }
      let componentHandle = new Component(instance);
      instance.definition().properties.forEach((prop) => {
        let propName = prop.name.replace(/-/g, "_");
        if (componentHandle[propName] !== void 0) {
          console.warn("Duplicated property name " + propName);
        } else {
          Object.defineProperty(componentHandle, propName, {
            get() {
              return instance.getProperty(prop.name);
            },
            set(value) {
              instance.setProperty(prop.name, value);
            },
            enumerable: true
          });
        }
      });
      instance.definition().callbacks.forEach((cb) => {
        let callbackName = cb.replace(/-/g, "_");
        if (componentHandle[callbackName] !== void 0) {
          console.warn("Duplicated callback name " + callbackName);
        } else {
          Object.defineProperty(componentHandle, cb.replace(/-/g, "_"), {
            get() {
              return function() {
                return instance.invoke(cb, Array.from(arguments));
              };
            },
            set(callback) {
              instance.setCallback(cb, callback);
            },
            enumerable: true
          });
        }
      });
      instance.definition().globals.forEach((globalName) => {
        if (componentHandle[globalName] !== void 0) {
          console.warn("Duplicated property name " + globalName);
        } else {
          let globalObject = /* @__PURE__ */ Object.create({});
          instance.definition().globalProperties(globalName).forEach((prop) => {
            let propName = prop.name.replace(/-/g, "_");
            if (globalObject[propName] !== void 0) {
              console.warn("Duplicated property name " + propName + " on global " + global);
            } else {
              Object.defineProperty(globalObject, propName, {
                get() {
                  return instance.getGlobalProperty(globalName, prop.name);
                },
                set(value) {
                  instance.setGlobalProperty(globalName, prop.name, value);
                },
                enumerable: true
              });
            }
          });
          instance.definition().globalCallbacks(globalName).forEach((cb) => {
            let callbackName = cb.replace(/-/g, "_");
            if (globalObject[callbackName] !== void 0) {
              console.warn("Duplicated property name " + cb + " on global " + global);
            } else {
              Object.defineProperty(globalObject, cb.replace(/-/g, "_"), {
                get() {
                  return function() {
                    return instance.invokeGlobal(globalName, cb, Array.from(arguments));
                  };
                },
                set(callback) {
                  instance.setGlobalCallback(globalName, cb, callback);
                },
                enumerable: true
              });
            }
          });
          Object.defineProperty(componentHandle, globalName, {
            get() {
              return globalObject;
            },
            enumerable: true
          });
        }
      });
      return Object.seal(componentHandle);
    }
  });
  return Object.seal(slint_module);
}
var EventLoop = class {
  #quit_loop = false;
  #terminationPromise = null;
  #terminateResolveFn;
  constructor() {
  }
  start(running_callback) {
    if (this.#terminationPromise != null) {
      return this.#terminationPromise;
    }
    this.#terminationPromise = new Promise((resolve) => {
      this.#terminateResolveFn = resolve;
    });
    this.#quit_loop = false;
    if (running_callback != void 0) {
      napi.invokeFromEventLoop(() => {
        running_callback();
        running_callback = void 0;
      });
    }
    const nodejsPollInterval = 16;
    let id = setInterval(() => {
      if (napi.processEvents() == napi.ProcessEventsResult.Exited || this.#quit_loop) {
        clearInterval(id);
        this.#terminateResolveFn(void 0);
        this.#terminateResolveFn = null;
        this.#terminationPromise = null;
        return;
      }
    }, nodejsPollInterval);
    return this.#terminationPromise;
  }
  quit() {
    this.#quit_loop = true;
  }
};
var globalEventLoop = new EventLoop();
function runEventLoop(runningCallback) {
  return globalEventLoop.start(runningCallback);
}
function quitEventLoop() {
  globalEventLoop.quit();
}
var private_api;
((private_api2) => {
  private_api2.mock_elapsed_time = napi.mockElapsedTime;
  private_api2.ComponentCompiler = napi.ComponentCompiler;
  private_api2.ComponentDefinition = napi.ComponentDefinition;
  private_api2.ComponentInstance = napi.ComponentInstance;
  private_api2.ValueType = napi.ValueType;
  private_api2.Window = napi.Window;
  private_api2.SlintBrush = napi.SlintBrush;
  private_api2.SlintRgbaColor = napi.SlintRgbaColor;
  private_api2.SlintSize = napi.SlintSize;
  private_api2.SlintPoint = napi.SlintPoint;
  private_api2.SlintImageData = napi.SlintImageData;
  function send_mouse_click(component, x, y) {
    component.component_instance.sendMouseClick(x, y);
  }
  private_api2.send_mouse_click = send_mouse_click;
  function send_keyboard_string_sequence(component, s) {
    component.component_instance.sendKeyboardStringSequence(s);
  }
  private_api2.send_keyboard_string_sequence = send_keyboard_string_sequence;
})(private_api || (private_api = {}));
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  ArrayModel,
  Brush,
  CompileError,
  Diagnostic,
  DiagnosticLevel,
  Model,
  RgbaColor,
  loadFile,
  private_api,
  quitEventLoop,
  runEventLoop
});
